
package edu.columbia.logging.file;

/**
 * Class to provide encoding/decoding capability for disk folder names
 *
 * Note: There's been reported problems using the '%' character within some browsers
 * on some OS's.  Thus using encodeNew is the best choice.
 */
public class EncoderDecoder {
	// NOTE: The encoding algorithm requires the '%' character to be first in the list since its in the
	//       replacement string encoding.
	private static final String[] delims =    {    "%",    "/",   "\\",    ":",    "<",    ">",    "?",    "#",    "[",    "]",    "$",    "!",    "&" };
	private static final String[] replsOld =  {  "%25",  "%2f",  "%5c",  "%3a",  "%3c",  "%3e",  "%3f",  "%23",  "%5b",  "%5d",  "%24",  "%21",  "%26" };
	private static final String[] replsNew =  { "x25x", "x2fx", "x5cx", "x3ax", "x3cx", "x3ex", "x3fx", "x23x", "x5bx", "x5dx", "x24x", "x21x", "x26x" };
	private static final String[] replsUnd =  {    "_",    "_",    "_",    "_",    "_",    "_",    "_",    "_",    "_",    "_",    "_",    "_",    "_" };
	
	/**
	 * Encode the string using the %XX special character replacement algorithm
	 * @param source
	 * @return encoded String or the original if encoding fails
	 */
	public static String encodeOld(String source) {
		return encodeSrcRepls(source, replsOld);
	}
	
	/**
	 * Encode the string using the xXXx special character replacement algorithm
	 * @param source
	 * @return encoded String or the original if encoding fails
	 */
	public static String encodeNew(String source) {
		return encodeSrcRepls(source, replsNew);
	}
	
	/**
	 * Encode the string using the '_' character to replace special characters
	 * @param source
	 * @return encoded String or the original if encoding fails
	 */
	public static String encodeUnd(String source) {
		return encodeSrcRepls(source, replsUnd);
	}
	
	private static String encodeSrcRepls(String source, String [] repls) {
		int len = repls[0].length();
		StringBuffer resultBuffer = new StringBuffer(source);
		for (int index = 0; index < delims.length; index++) {
			String del = delims[index];
			int pos = resultBuffer.indexOf(del);
			while (pos != -1) {
				resultBuffer.deleteCharAt(pos);
				resultBuffer.insert(pos, repls[index]);
				pos = resultBuffer.indexOf(del, pos + len);
			}
		}
		return resultBuffer.toString();
	}

	/**
	 * Decode the string using the %XX encoding algorithm
	 * @param source
	 * @return decoded String or the original if decoding fails
	 */
	public static String decodeOld(String source) {
		return decodeSrcRepls(source, replsOld);
	}
	
	/**
	 * Decode the string using the xXXx encoding algorithm
	 * @param source
	 * @return decoded String or the original if decoding fails
	 */
	public static String decodeNew(String source) {
		return decodeSrcRepls(source, replsNew);
	}
	
	/**
	 * Decode the string trying the %XX algorithm first and if not used then the xXXx algorithm
	 * @param source
	 * @return decoded String or the original if decoding fails
	 */
	public static String decodeUnk(String source) {
		String str;
		
		str = decodeSrcRepls(source, replsOld);
		if (str.equals(source)) {
			str = decodeSrcRepls(source, replsNew);
		}
		return str;
	}
	
	private static String decodeSrcRepls(String source, String [] repls) {
		int len = repls[0].length();
		StringBuffer resultBuffer = new StringBuffer(source);
		for (int index = 0; index < repls.length; index++) {
			String repl = repls[index];
			int pos = resultBuffer.indexOf(repl);
			while (pos != -1) {
				resultBuffer.delete(pos, pos+len);
				resultBuffer.insert(pos, delims[index]);
				pos = resultBuffer.indexOf(repl, pos + 1);
			}
		}
		return resultBuffer.toString();
	}
	
//	public static void main(String[] args) {
//		String original = 
//			"pct=%,fslash=/,bslash=\\,colon=:,gt=>,ques=?,sharp=#,lb=[,rb=],dollar=$,exclam=!,amp=&,gt2=>,fslash2=/";
//		String encoded =  EncoderDecoder.encodeNew(original);
//		System.out.println("encoded string is: " + encoded);
//		System.out.println("decoded string is: " + EncoderDecoder.decodeUnk(encoded));
//	}
}
