package edu.columbia.logging.file;
/*
 * parser for parsing chat transcript
 * author shruti
 */

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.URL;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import sun.nio.cs.ext.MacHebrew;

import com.sun.org.apache.xpath.internal.XPathAPI;


public class FileChatTranscript implements IChatTranscript {
	/** The name of the META tag for the hostname */
	public static final String META_HOSTNAME = "sametime:hostname";  
	
	/** The name of the META tag for the provider Id */
	public static final String META_PROVIDER_ID = "sametime:providerId";  
	
	/** The name of the META tag for the provider Id */
	public static final String META_USERNAME = "sametime:username";  
	
	/** The name of the META tag for the chat initiator */
	public static final String META_INITIATOR = "sametime:initiator";  
	
	/** The name of the META tag for the chat topic */
	public static final String META_CHAT_TOPIC = "sametime:chatTopic";  
	
	/** The name of the META tag for the chat creattionTime */
	public static final String META_CREATE_TIME = "sametime:creationTime";  
	
	/** The name of the META tag for the last activity time */
	public static final String META_LAST_ACTIVITY_TIME = "sametime:lastActivityTime";  
	
	/** The name of the class value for the time stamp */
	public static final String CLASS_TIMESTAMP = "showTimestamp tsDisplay";  
	
	private static final String TEMP_SAVE_FILE_PREFIX = "sametimeSaveTrans";  
	
	private static final String TEMP_SAVE_FILE_SUFFIX = ".htm";  
	
	private class MetaInfoReader extends DefaultHandler {
		public String hostname = null;
		public String providerId = null;
		public String username = null;
		public String initiator = null;
		public String topic = null;
		public Date creationTime = null;
		public Date lastActivityTime = null;
		
		public void parseMetaInfo(InputSource inputSrc ) {
			try {
				SAXParserFactory factory = SAXParserFactory.newInstance();
				factory.setNamespaceAware(true);
				SAXParser parser = factory.newSAXParser();
				parser.parse(inputSrc, this);
			}
			catch (Throwable t) {
				if(logger.isLoggable(Level.FINE)) {
					logger.log(Level.FINE, 
	 					       m_file.getAbsolutePath()+ t.getMessage());
				}
			}
		}
		
		public void startElement(String uri, String localName, String qName, Attributes attrs) throws SAXException {
			if (qName.equals("meta")) {  
				int index = attrs.getIndex("name");  
				if(index >= 0) {
					String which = attrs.getValue(index);
					if(which.equals(META_HOSTNAME)) {
						index = attrs.getIndex("content");  
						//-------------------------------------------
						// Because earlier used "contents" rather
						// than content, check for that too
						//-------------------------------------------
						if(index < 0) {
							index = attrs.getIndex("contents");  
						}
						if(index >= 0) {
							hostname = attrs.getValue(index);
						}
					}
					if(which.equals(META_PROVIDER_ID)) {
						index = attrs.getIndex("content");  
						//-------------------------------------------
						// Because earlier used "contents" rather
						// than content, check for that too
						//-------------------------------------------
						if(index < 0) {
							index = attrs.getIndex("contents");  
						}
						if(index >= 0) {
							providerId = attrs.getValue(index);
						}
					}
					else if(which.equals(META_USERNAME)) {
						index = attrs.getIndex("content");  
						//-------------------------------------------
						// Because earlier used "contents" rather
						// than content, check for that too
						//-------------------------------------------
						if(index < 0) {
							index = attrs.getIndex("contents");  
						}
						if(index >= 0) {
							username = attrs.getValue(index);
						}
					}
					else if(which.equals(META_INITIATOR)) {
						index = attrs.getIndex("content");  
						if(index >= 0) {
							initiator = EncoderDecoder.decodeOld(attrs.getValue(index));
							ChatParserANDWriter.userI = initiator.substring(0, initiator.indexOf('@'));
						}
					}
					else if(which.equals(META_CHAT_TOPIC)) {
						index = attrs.getIndex("content");  
						if(index >= 0) {
							topic = EncoderDecoder.decodeOld(attrs.getValue(index));
						}
					}
					else if(which.equals(META_CREATE_TIME)) {
						index = attrs.getIndex("content");  
						if(index >= 0) {
							String timeString = attrs.getValue(index);
							try
							{
								DateFormat format;
								if(timeHasSeconds(timeString)) {
									format = s_longDateFormat;
								}
								else {
									format = s_longDateFormatNoSecs;
								}
								creationTime = format.parse(timeString);
							}
							catch(ParseException pfe)
							{
								creationTime = null;
							}
						}
					}
					else if(which.equals(META_LAST_ACTIVITY_TIME)) {
						index = attrs.getIndex("content");  
						if(index >= 0) {
							String timeString = attrs.getValue(index);
							try
							{
								DateFormat format;
								if(timeHasSeconds(timeString)) {
									format = s_longDateFormat;
								}
								else {
									format = s_longDateFormatNoSecs;
								}
								lastActivityTime = format.parse(timeString);
							}
							catch(ParseException pfe)
							{
								lastActivityTime = null;
							}
						}
					}
				}			
			}
		}
	}
	
	private class ChatCountParser extends DefaultHandler {
		private Date creationTime;
		private String lastTimestamp;
		private boolean inTimestamp;
		
		public ChatCountParser(Date creationTime) {
			this.creationTime = creationTime;
			inTimestamp = false;
		}
		
		public void startDocument() throws SAXException {
			// TODO Auto-generated method stub
			super.startDocument();
		}
		
		public int parse(InputSource inputSrc ) {
			System.out.println("IN DOM pre");
			domparser(inputSrc);
			System.out.println(m_file.getAbsoluteFile());
			/*try {
				SAXParserFactory factory = SAXParserFactory.newInstance();
				factory.setNamespaceAware(true);
				
				SAXParser parser = factory.newSAXParser();
				parser.parse(inputSrc, this);
			}
			catch (Throwable t) {
				if(logger.isLoggable(Level.INFO)) {
					logger.log(Level.INFO, 
 					            m_file.getAbsolutePath() + t.getMessage());
				}
				t.printStackTrace();
			}*/
			return count;
		}
		private String xmlToString(Node node) {
			try {
			Source source = new DOMSource(node);
			StringWriter stringWriter = new StringWriter();
			Result result = new StreamResult(stringWriter);
			TransformerFactory factory = TransformerFactory.newInstance();
			Transformer transformer = factory.newTransformer();
			transformer.transform(source, result);
			return stringWriter.getBuffer().toString();
			} catch (TransformerConfigurationException e) {
			e.printStackTrace();
			} catch (TransformerException e) {
			e.printStackTrace();
			}
			return null;
			}
		
		int chatbytes = 0;
		public void domparser(InputSource inputSrc){
			System.out.println("IN DOM");
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory
					.newInstance();

			try {
					DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
					//Document doc = docBuilder.parse("C:\\Documents and Settings\\Administrator\\SametimeTranscripts\\ashwoodb@us.ibm.com\\20070329\\ashwoodblog.html");
					Document doc = docBuilder.parse(inputSrc);
					Element documentRoot = doc.getDocumentElement();
					System.out.println("IN DOM1");
					NodeList matchedNodes=null;
					String query="";
					try{
					query = "//*/div[@title]";
					matchedNodes = com.sun.org.apache.xpath.internal.XPathAPI.selectNodeList(documentRoot, query);
					System.out.println("DOM COUNT: testing ");
					System.out.println("DOM COUNT: " + matchedNodes);
					System.out.println("DOM COUNT: " + matchedNodes.getLength());
					}catch(Exception e){
						System.out.println("ERROR" + e.getMessage());
						e.printStackTrace();
						
					}
					count = matchedNodes.getLength();
					System.out.println("IN DOM2");		
					query = "//*/div[@class='myself']";
					matchedNodes = XPathAPI.selectNodeList(documentRoot, query);
					myself = myself + matchedNodes.getLength();
					System.out.println("IN DOM3");
					query = "//*/div[@class='myselfCont']";
					matchedNodes = XPathAPI.selectNodeList(documentRoot, query);
					myself = myself + matchedNodes.getLength();
				//	System.out.println("myself "+myself);
					System.out.println("IN DOM4");
					query = "//*/div[@class='other']";
					matchedNodes = XPathAPI.selectNodeList(documentRoot, query);
				//	other = other + matchedNodes.getLength();
					System.out.println("IN DOM5");
					query = "//*/div[@class='otherCont']";
					matchedNodes = XPathAPI.selectNodeList(documentRoot, query);
					other = other + matchedNodes.getLength();
					System.out.println("other: "+other);
					
					
//					query = "//*/div[@class='message']";
//					matchedNodes = XPathAPI.selectNodeList(documentRoot, query);	
//					System.out.println("message matched node: "+matchedNodes.getLength());
//					for (int j= 0; j < matchedNodes.getLength(); j++) {
//						Node node = matchedNodes.item(j);
//						System.out.println("node: "+node.toString());
//						if (node != null) {		
//							
//							if(node.getFirstChild().getFirstChild().getNodeValue() != null && node.getFirstChild().getFirstChild().getNodeValue().length() > 0 && node.getFirstChild().getFirstChild().getNodeValue() != ""){
//								String chat = node.getFirstChild().getFirstChild().getNodeValue();
//								System.out.println("chat: "+chat);
//								chatbytes = chatbytes + chat.getBytes().length;
//							}else if(node.getFirstChild().getFirstChild().getFirstChild().getNodeValue() != null && node.getFirstChild().getFirstChild().getFirstChild().getNodeValue().length() > 0 && node.getFirstChild().getFirstChild().getFirstChild().getNodeValue() != ""){
//								String chat = node.getFirstChild().getFirstChild().getFirstChild().getNodeValue();
//								System.out.println("chat: "+chat);
//								chatbytes = chatbytes + chat.getBytes().length;
//							}
//								
//							System.out.println("otchatbytesher: "+chatbytes);
//						}
//					}
//					
//					query = "//*/div[@class='messageCont']";
//					matchedNodes = XPathAPI.selectNodeList(documentRoot, query);					
//					for (int j= 0; j < matchedNodes.getLength(); j++) {
//						Node node = matchedNodes.item(j);
//						
//						if (node != null) {		
//							String chat = node.getFirstChild().getFirstChild().getNodeValue();
//							System.out.println("chat: "+chat);
//							if(chat != null && chat.length() > 0 && chat != ""){
//								chatbytes = chatbytes + chat.getBytes().length;
//							}else{
//								chat = node.getFirstChild().getFirstChild().getFirstChild().getNodeValue();
//								System.out.println("chat: "+chat);
//								chatbytes = chatbytes + chat.getBytes().length;
//							}
//								
//							System.out.println("otchatbytesher: "+chatbytes);
//						}
//					}
//					System.out.println("otchatbytesher: "+chatbytes);
//		    		
//		    		Integer chatt = (Integer)ChatParserANDWriter.user_bytes.get(userid);	
//					 if (chatt == null)
//						 ChatParserANDWriter.user_bytes.put(userid, new Integer(chatbytes));
//					 else{
//						 int chatb = chatt.intValue() + chatbytes;
//						 ChatParserANDWriter.user_bytes.put(userid, new Integer(chatb));
//					 }
					
					query = "//*/div[@class='showTimestamp tsDisplay']";
					matchedNodes = XPathAPI.selectNodeList(documentRoot, query);
					
					for (int j= 0; j < matchedNodes.getLength(); j++) {
						
						Node node = matchedNodes.item(j);
						System.out.println("matched node: " + xmlToString(node));
						//NamedNodeMap attrs = node.getAttributes();
						if (node != null) {		
							lastTimestamp = node.getFirstChild().getFirstChild().getNodeValue();
							System.out.println("lastTimestamp = "+lastTimestamp);	
					//		System.out.println("OK");
							Date recenttime = getLastActivityTime();
							if(first == null && isStartAssigned == false){
								first = recenttime;
								last = recenttime;
								isStartAssigned = true;
							}
							//System.out.println("recenttime: "+recenttime+ " last: "+last);
							if(recenttime != null && last != null && first != null){
							if(!(j==matchedNodes.getLength()-1) && ((recenttime.getHours()*60 + recenttime.getMinutes()) - (last.getHours()*60 + last.getMinutes()) < 30))
								last = recenttime;
							else {
								cdo.start = first.getHours()+":"+first.getMinutes()+":"+first.getSeconds();
								cdo.end = last.getHours()+":"+last.getMinutes()+":"+last.getSeconds();
								cdo.other = other;
								cdo.me = myself;
								cdo.length = (last.getHours()*60 + last.getMinutes()) - (first.getHours()*60 + first.getMinutes());
							//	System.out.println("OYE:: date: "+dateName+ " start: "+ cdo.start + " end: "+ cdo.end + " other: "+cdo.other + " me: "+ cdo.me + " length: "+ cdo.length);
								if(cdo.length < 0){
								//	System.out.println("In file: "+m_file.getAbsolutePath());
								//	System.out.println((last.getHours()*60 + last.getMinutes())+ " "+(first.getHours()*60 + first.getMinutes()));
									//System.out.println("OYE:: "+ last.getHours()+" " + last.getMinutes() +" "+first.getHours() +" "+ first.getMinutes() +" +date: "+dateName+ " start: "+ cdo.start + " end: "+ cdo.end + " other: "+cdo.other + " me: "+ cdo.me + " length: "+ cdo.length);
									cdo.length = ((last.getHours()+24)*60 + last.getMinutes()) - (first.getHours()*60 + first.getMinutes());
								//	System.out.println("OYE:: "+ last.getHours()+" " + last.getMinutes() +" "+first.getHours() +" "+ first.getMinutes() +" +date: "+dateName+ " start: "+ cdo.start + " end: "+ cdo.end + " other: "+cdo.other + " me: "+ cdo.me + " length: "+ cdo.length);
								}
								SortedMap time_entries = (SortedMap)ChatParserANDWriter.storeuserchattimeinfo.get(userid);
								 if (time_entries == null)
						            	ChatParserANDWriter.storeuserchattimeinfo.put(userid, time_entries=new TreeMap());
								 time_entries.put( dateName,cdo);
							//	ChatParserANDWriter.storeuserchattimeinfo.put("gandhish@us.ibm.com", cdo);
								ChatParserANDWriter.chatdetailobjectlist.add(cdo);
								other = 0;
								myself = 0;
								first = null;
								last = null;
								isStartAssigned = false;
							}
							}
//							for(int i=0; i<attrs.getLength(); i++) {
////							System.out.println(attrs.item(i).getNodeName() + " emm " + attrs.item(i).getNodeValue());
//							}
						}						
					}
					
					query = "//*/div[@class='message']";
					
					matchedNodes = XPathAPI.selectNodeList(documentRoot, query);	
					if(matchedNodes != null){
						System.out.println("Lenght of matchedNodes: "+matchedNodes.getLength());
					for (int j= 0; j < matchedNodes.getLength(); j++) {
					

						if(matchedNodes.item(j)!=null && matchedNodes.item(j).getNodeType()==Node.TEXT_NODE){
							chatbytes = chatbytes + matchedNodes.item(j).getNodeValue().getBytes().length;
						//	System.out.println("message matched node: "+matchedNodes.item(j).getNodeValue().getBytes().length);													
						}else if(matchedNodes.item(j).getFirstChild()!=null &&matchedNodes.item(j).getFirstChild().getNodeType()==Node.TEXT_NODE){
							chatbytes = chatbytes + matchedNodes.item(j).getFirstChild().getNodeValue().getBytes().length;
						//	System.out.println("1message matched node: "+matchedNodes.item(j).getFirstChild());
						}else if(matchedNodes.item(j).getFirstChild().getFirstChild()!=null && matchedNodes.item(j).getFirstChild().getFirstChild().getNodeType()==Node.TEXT_NODE){
							chatbytes = chatbytes + matchedNodes.item(j).getFirstChild().getFirstChild().getNodeValue().getBytes().length;
						//	System.out.println("2message matched node: "+matchedNodes.item(j).getFirstChild().getFirstChild());
						}else if(matchedNodes.item(j).getFirstChild().getFirstChild().getFirstChild()!=null &&matchedNodes.item(j).getFirstChild().getFirstChild().getFirstChild().getNodeType()==Node.TEXT_NODE){
							chatbytes = chatbytes + matchedNodes.item(j).getFirstChild().getFirstChild().getFirstChild().getNodeValue().getBytes().length;
						//	System.out.println("3message matched node: "+matchedNodes.item(j).getFirstChild().getFirstChild().getFirstChild());
						}
			
					}
					}
					query = "//*/div[@class='messageCont']";
					
					matchedNodes = XPathAPI.selectNodeList(documentRoot, query);	
					if(matchedNodes != null){
						System.out.println("Lenght of matchedNodes: "+matchedNodes.getLength());
						
					for (int j= 0; j < matchedNodes.getLength(); j++) {
						
						System.out.println("Lenght of matchedNodes J: "+j);
						System.out.println("1 " +matchedNodes.item(j));
						System.out.println("2" +matchedNodes.item(j).getNodeType());
						System.out.println("2" +matchedNodes.item(j).getNodeType());
						System.out.println("3" +matchedNodes.item(j).getFirstChild().getFirstChild().getFirstChild());
						System.out.println("4" +matchedNodes.item(j).getFirstChild().getFirstChild().getFirstChild().getNodeType());
						System.out.println("5" +matchedNodes.item(j).getFirstChild().getFirstChild().getNodeValue().getBytes().length);
						System.out.println("6" +matchedNodes.item(j).getFirstChild().getFirstChild().getNodeValue());
						if(matchedNodes.item(j)!=null && matchedNodes.item(j).getNodeType()==Node.TEXT_NODE){
							chatbytes = chatbytes + matchedNodes.item(j).getNodeValue().getBytes().length;
							//System.out.println("message matched node: "+matchedNodes.item(j).getNodeValue().getBytes().length);													
						}else if(matchedNodes.item(j).getFirstChild()!=null &&matchedNodes.item(j).getFirstChild().getNodeType()==Node.TEXT_NODE){
							chatbytes = chatbytes + matchedNodes.item(j).getFirstChild().getNodeValue().getBytes().length;
					//		System.out.println("1message matched node: "+matchedNodes.item(j).getFirstChild());
						}else if(matchedNodes.item(j).getFirstChild().getFirstChild()!=null && matchedNodes.item(j).getFirstChild().getFirstChild().getNodeType()==Node.TEXT_NODE){
							chatbytes = chatbytes + matchedNodes.item(j).getFirstChild().getFirstChild().getNodeValue().getBytes().length;
						//	System.out.println("2message matched node: "+matchedNodes.item(j).getFirstChild().getFirstChild());
						}else if(matchedNodes.item(j).getFirstChild().getFirstChild().getFirstChild()!=null && matchedNodes.item(j).getFirstChild().getFirstChild().getFirstChild().getNodeType()==Node.TEXT_NODE){
							chatbytes = chatbytes + matchedNodes.item(j).getFirstChild().getFirstChild().getFirstChild().getNodeValue().getBytes().length;
							//System.out.println("3message matched node: "+matchedNodes.item(j).getFirstChild().getFirstChild().getFirstChild());
						}
						
					}
					}
	
					System.out.println("otchatbytesher: "+chatbytes);
					Integer chatt = (Integer)ChatParserANDWriter.user_bytes.get(userid);	
					 if (chatt == null)
						 ChatParserANDWriter.user_bytes.put(userid, new Integer(chatbytes));
					 else{
						 int chatb = chatt.intValue() + chatbytes;
						 ChatParserANDWriter.user_bytes.put(userid, new Integer(chatb));
					 }
					
					} catch (ParserConfigurationException e) {
						System.out.println(m_file.getAbsolutePath()+" - " + e.getMessage());
					// TODO Auto-generated catch block
					e.printStackTrace();
					} catch (SAXException e) {
					// TODO Auto-generated catch block
						System.out.println(m_file.getAbsolutePath()+" - " + e.getMessage());
						
					} catch (IOException e) {
						System.out.println(m_file.getAbsolutePath()+" - " + e.getMessage());
					// TODO Auto-generated catch block
						
					e.printStackTrace();
					} catch (TransformerException e) {
						System.out.println(m_file.getAbsolutePath()+" - " + e.getMessage());
					// TODO Auto-generated catch block
					e.printStackTrace();
					}
					catch (Exception e) {
						System.out.println(m_file.getAbsolutePath()+" - " + e.getMessage());
					// TODO Auto-generated catch block
					e.printStackTrace();
					}
				
		}
		
		
		
		int count=0;
		ChatDetailObject cdo = new ChatDetailObject();
		int myself = 0;
		int other = 0;
		int length = 0;
		String start="", end="";
		public void startElement(String uri, String localName, String qName, Attributes attrs) throws SAXException {
		//	System.out.println(qName);
			//logger.info("In interesting1");
			if (qName.equals("div")) {  
				int index = attrs.getIndex("title");  
				if(index >= 0) {
					count++;
				}
				
				index = attrs.getIndex("class");  
				if(index >= 0) {
					String which = attrs.getValue(index);
					if(which.equals(CLASS_TIMESTAMP)) {
						inTimestamp = true;
					}
					if(which.equals("myself") || which.equals("myselfCont")) {
						myself++;
					}
					if(which.equals("other")  || which.equals("otherCont")) {
						other++;
					}
				}
				
			}
			
		
		}
		
		public void endElement(String uri, String localName, String qName) throws SAXException {
		}
		
		public void endDocument() throws SAXException {
		
			super.endDocument();
			if(first != null && last != null){
			if(isStartAssigned == true && first != null && last != null){
				cdo.start = first.getHours()+":"+first.getMinutes()+":"+first.getSeconds();
				cdo.end = last.getHours()+":"+last.getMinutes()+":"+last.getSeconds();
				cdo.other = other;
				cdo.me = myself;
				cdo.length = (last.getHours()*60 + last.getMinutes()) - (first.getHours()*60 + first.getMinutes());
			//	System.out.println("OYE:: date: "+dateName+ " start: "+ cdo.start + " end: "+ cdo.end + " other: "+cdo.other + " me: "+ cdo.me + " length: "+ cdo.length);
				
				SortedMap time_entries = (SortedMap)ChatParserANDWriter.storeuserchattimeinfo.get(userid);
				 if (time_entries == null)
		            	ChatParserANDWriter.storeuserchattimeinfo.put(userid, time_entries=new TreeMap());
				 time_entries.put( dateName,cdo);
			//	ChatParserANDWriter.storeuserchattimeinfo.put("gandhish@us.ibm.com", cdo);
				 ChatParserANDWriter.chatdetailobjectlist.add(cdo);
				other = 0;
				myself = 0;
				first = null;
				last = null;
				isStartAssigned = false;
			}
			}
		}
		
		Date first=null, last=null;
		boolean isStartAssigned = false;
		public void characters(char[] ch, int start, int length) throws SAXException {
		//	System.out.println("here1");
			if(inTimestamp) {
				lastTimestamp = new String(ch, start, length);
			//	System.out.println("Parsing timestamp: "+lastTimestamp);
				Date recenttime = getLastActivityTime();
				if(first == null && isStartAssigned == false){
					first = recenttime;
					last = recenttime;
					isStartAssigned = true;
				}
				//System.out.println("recenttime: "+recenttime+ " last: "+last);
				if(recenttime != null && last != null && first != null){
				if((recenttime.getHours()*60 + recenttime.getMinutes()) - (last.getHours()*60 + last.getMinutes()) < 30)
					last = recenttime;
				else {
					cdo.start = first.getHours()+":"+first.getMinutes()+":"+first.getSeconds();
					cdo.end = last.getHours()+":"+last.getMinutes()+":"+last.getSeconds();
					cdo.other = other;
					cdo.me = myself;
					cdo.length = (last.getHours()*60 + last.getMinutes()) - (first.getHours()*60 + first.getMinutes());
				//	System.out.println("OYE:: date: "+dateName+ " start: "+ cdo.start + " end: "+ cdo.end + " other: "+cdo.other + " me: "+ cdo.me + " length: "+ cdo.length);
					
					SortedMap time_entries = (SortedMap)ChatParserANDWriter.storeuserchattimeinfo.get(userid);
					 if (time_entries == null)
			            	ChatParserANDWriter.storeuserchattimeinfo.put(userid, time_entries=new TreeMap());
					 time_entries.put( dateName,cdo);
				//	ChatParserANDWriter.storeuserchattimeinfo.put("gandhish@us.ibm.com", cdo);
					ChatParserANDWriter.chatdetailobjectlist.add(cdo);
					other = 0;
					myself = 0;
					first = null;
					last = null;
					isStartAssigned = false;
				}
				}
			//	System.out.println("lastTimestamp is " + lastTimestamp);
				inTimestamp = false;
			}
		}
		
		public Date getLastActivityTime() {
			Date result = null;
			if(lastTimestamp != null) {
				result = convertTimestamp(creationTime, lastTimestamp);
				//System.out.println("result is " + result);
			}
			
			return result;
		}
	}
	
	/**
	 * Class to represent key used for mapping between community meta information
	 * and the community ID
	 *
	 */
	private static class CommunityMetaInfoKey
	{
		String providerId;
		String host;
		String username;
		
		/**
		 * Construct this CommunityMetaInfoKey
		 * @param providerId
		 * @param host
		 * @param username
		 */
		public CommunityMetaInfoKey(String providerId, String host, String username)
		{
			this.providerId = providerId;
			this.host = host;
			this.username = username;
		}
		
		/* (non-Javadoc)
		 * @see java.lang.Object#equals(java.lang.Object)
		 */
		public boolean equals(Object anObject) {
			boolean result = false;
			if(anObject instanceof CommunityMetaInfoKey) {
				CommunityMetaInfoKey another = (CommunityMetaInfoKey) anObject;
				result =
					another.providerId.equals(providerId) &&
					another.host.equals(host) &&
					another.username.equals(username);				
			}
			
			return result;
		}
	}
	
	/**
	 * Class to help us map between the community related meta information
	 * we store in the transcript and a community ID.
	 * 
	 */
	private static class CommunityMetaInfoToCommunityMap
	{
		private HashMap map;
		private CommunityMetaInfoKey findKey;
			
		/**
		 * Construct this CommunityMetaInfoToCommunityMap
		 */
		public CommunityMetaInfoToCommunityMap() {
			map = new HashMap();
			findKey = new CommunityMetaInfoKey("", "", "");   //$NON-NLS-2$ //$NON-NLS-3$
		}
		
		/**
		 * Try to find an entry with the given community meta information
		 * @param providerId
		 * @param host
		 * @param username
		 * @return community ID or <code>null</code> if not found
		 */
		public String findEntry(String providerId, String host, String username) {
			String result = null;
			synchronized(this)
			{
				//---------------------------------------------------
				// Since we expect to get here a lot, we reuse the
				// same key when doing our search.  We are 
				// synchronizing on this so we should be safe
				// doing this.
				//---------------------------------------------------
				findKey.providerId = providerId;
				findKey.host = host;
				findKey.username = username;
				result = (String) map.get(findKey);
			}
			
			return result;
		}
		
		/**
		 * Add an entry with the given community meta information and communityID
		 * @param providerId
		 * @param host
		 * @param username
		 * @param community ID
		 */
		public void addEntry(String providerId, 
				             String host, 
				             String username,
				             String communityID) {
			CommunityMetaInfoKey key = 
				new CommunityMetaInfoKey(providerId, host, username);
			synchronized(this)
			{
				//---------------------------------------------------
				// Make sure not in there already since it could
				// happen in a timing window
				//---------------------------------------------------
				if(map.get(key) == null) {
					map.put(key, communityID);
				}
			}
		}
	}
	
	private static Logger logger = Logger.getLogger(FileChatTranscript.class.getPackage().getName());
	
	private static final String LONG_DATE_FORMAT = "yyyyMMdd-HHmmss (Z)";  
	private static final String LONG_DATE_FORMAT_NOSECS = "yyyyMMdd-HHmm (Z)";  
	private static final String TIME_FORMAT_12 = "hh:mm:ss aa";  
	private static final String TIME_FORMAT_24 = "hh:mm:ss";
	private static final String TAG_IMG = "img";  
	private static final String TAG_IMG_PARM = "src";  
	private static CommunityMetaInfoToCommunityMap s_metaInfoToCommunityIdMap = 
		new CommunityMetaInfoToCommunityMap();
	
	private static DateFormat s_longDateFormat = new SimpleDateFormat(LONG_DATE_FORMAT);
	private static DateFormat s_longDateFormatNoSecs = new SimpleDateFormat(LONG_DATE_FORMAT_NOSECS);
	private static DateFormat s_timeFormat12 = new SimpleDateFormat(TIME_FORMAT_12);
	private static DateFormat s_timeFormat24 = new SimpleDateFormat(TIME_FORMAT_24);
	
	private File m_file;
	
	
	private String m_headerText = null;
	private String m_footerText = null;
	private StringBuffer m_contents = null;
	
	private Calendar m_calendarStart;
	private Calendar m_calendarLast;
	private int m_startDayOfYear;
	private Date m_date;
	private Date m_lastActivity;
	private String m_chatPartnerID = null;
	private boolean m_chatPartnerExt = false;
	private String m_chatPartnerName = null;
	private String[] m_metaInfo;
	private String m_initiator;
	private String m_chatTopic;
	private String m_conversationID;
	private String m_communityID;
	private boolean m_parsedMetaInfo;
	private boolean m_parsedTimestamps;
	private boolean m_existingFile = false;
	private boolean m_SavePersonProperties = false;
	private boolean m_HaveSavedPersonProperties = false;
	private boolean m_HasContentsChanged = false;
	private boolean m_HasLoadedTranscript = false;
	private ArrayList m_CopyURLs = new ArrayList();
	private IChatPersonHistory m_chatPersonHistory = null;
	
	private static final String START_CONTENT = "<!--START CONTENT-->";  
	private static final String END_CONTENT = "<!--END CONTENT-->";  
	private static final String NEWLINE = "\r\n";  
	
	/**
	 * Construct this FileChatTranscript using the current date
	 * @param file the file the contents are going into
	 */
	public FileChatTranscript(File file) {
		this(file, new Date());
	}
	
	/**
	 * Construct this FileChatTranscript with the given date
	 * @param file the file the contents are going into
	 * @param date the date associated with tihs transcripts
	 */
	public FileChatTranscript(File file, Date date) {
		m_file = file;
		m_date = date;
		m_lastActivity = date;
		m_calendarStart = new GregorianCalendar();
		m_calendarStart.setTime(date);
		m_startDayOfYear = m_calendarStart.get(Calendar.DAY_OF_YEAR);
		m_calendarLast = new GregorianCalendar();
		m_calendarLast.setTime(date);
	}
	
	public void loadTranscript() {
		//
		// If we've already been initialized we don't want to load again
		//
		if ((m_contents != null) || (m_headerText != null) || (m_footerText != null)) {
			return;
		}
		if (!m_HasLoadedTranscript && (m_file != null) && m_file.exists()) {
			m_existingFile = true;
			m_HasLoadedTranscript = true;
			try {
				FileInputStream reader = new FileInputStream(m_file);
				InputStreamReader isr = new InputStreamReader(reader, "UTF-8");				  
				BufferedReader br = new BufferedReader(isr);
				String line = "";  
				boolean isContents = false;
				boolean isTail = false;
				
				StringBuffer header = new StringBuffer();
				StringBuffer footer = new StringBuffer();
				m_contents = new StringBuffer();
				while ((line = br.readLine()) != null) {
					if (line.compareTo(START_CONTENT) == 0) {
						isContents = true;
						continue;
					} 
					else if (line.compareTo(END_CONTENT) == 0) {
						isContents = false;
						isTail = true;
						continue;
					}
					if (!isContents && !isTail) {
						header.append(line);
					}
					else if (isContents) {
						m_contents.append(line);
					}
					else if (isTail) {
						footer.append(line);
					}
				}
				br.close();
				m_headerText = header.toString();
				m_footerText = footer.toString();
				
//				String strNewContents = unlocalizeURLs();
//				m_contents = null;
//				m_contents = new StringBuffer();
//				m_contents.append(strNewContents);
				
				header = null;
				footer = null;
			} catch (IOException e) {
				m_contents = null;
				m_headerText = null;
				m_footerText = null;
			}
		}
	}

	/**
	 * Get the date of this chat
	 * @return date
	 */
	public Date getDate() {
		if (m_date == null) {
			if (!m_parsedMetaInfo && m_existingFile) {
				parseMetaInfo();	
			}
		}
		return m_date;
	}
	
	 /**
     * Get # of times you chat with a user per day.
     * @return
     * 
     */
	String userid="";
	String dateName="";
 	public int getPerDayChatCount(String date,String user) {
 		userid = user;
 		dateName=date;
 		int count = parseChatCount();
 		return count;
	}	

 	
	
    /**
     * Get the date/time of last activity
     * @return
     * @deprecated - Should not be used and eventually will be removed
     */
 	public Date getLastActivity() {
 	
		if((m_lastActivity == null) || (m_date.equals(m_lastActivity))) {
		
			if(!m_parsedTimestamps && m_existingFile) {
			
				parseTimestamps();
			}
		}
		return m_lastActivity;
	}	

    /**
     * Get the initiator of this chat message
     * @deprecated - Should not be used and eventually will be removed
     */
	public String getChatInitiator() {
		if(m_initiator == null) {
			if(!m_parsedMetaInfo && m_existingFile) {
				parseMetaInfo();
			}
		}
		return m_initiator;
	}
	
	/**
	 * Set the chat initiator ID 
	 * @param userId the userId of the chat initiator
	 */
	public void setChatInitiator(String userId) {
		m_initiator = userId;
	}

	
    /**
     * @return String[] recipient with one entry non-null for a one-on-one chat and null for a multi-person chat
     * @deprecated - Should not be used and eventually will be removed
     */
	public String[] getChatRecipients() {
		return new String [] { m_chatPartnerID };
	}
	
    /**
     * Return the interface regarding the individual this transcript is from
     * @return
     */
    public IChatPersonHistory getChatPersonHistory() {
    	return m_chatPersonHistory;
    }
    
    public void setChatPersonHistory(IChatPersonHistory icph) {
    	m_chatPersonHistory = icph;
    }
    
    protected String getChatPartnerID() {
		return m_chatPartnerID;
	}
	
	protected boolean getChatPartnerExt() {
		return m_chatPartnerExt;
	}
	
	protected String getChatPartnerName() {
		return m_chatPartnerName;
	}

	protected String getConversationID() {
		return m_conversationID;
	}
	
	/**
	 * Set the conversationID associated with this transcript, for later retrieval
	 * @param conversationID
	 */
	protected void setConversationID(String conversationID) {
		m_conversationID = conversationID;
	}
	
	/**
	 * Get the chat topic
	 * @return the chat topic
	 */
	public String getChatTopic() {
		if(m_chatTopic == null) {
			if(!m_parsedMetaInfo && m_existingFile) {
				parseMetaInfo();	
			}
		}
		return m_chatTopic;
	}

	/**
     * Set the topic of this chat message
     * @param topic the topic
     */ 
	public void setChatTopic(String topic) {	
		m_chatTopic = topic;
	}

	/**
	 * Answer whether the chat is n-way
	 * @return <code>true</code> if n-way, <code>false</code> otherwise
	 */
	public boolean isChatNWay() {
		boolean result;
		
		//
		// If there is no chat partner it's an NWay chat
		//
		if (m_chatPartnerID == null) {
			result = true;
		} else {
			result = false;
		}
		return result;
	}
	
	/**
	 * Return the meta info as separate elements
	 * @return META element strings
	 */
	public String[] getMetaInfo() {
		if (m_metaInfo == null) {
			//
			// We want to retain the previous meta data where appropriate and if it hasn't yet
			// been parsed then we are using defaults which are not appropriate.
			//
			if (!m_parsedMetaInfo && m_existingFile) {
				parseMetaInfo();
			}
			
			
		}
		return m_metaInfo;
	}
	
	/** 
     * Get the contents this chat message
     * @return the contents (typically HTML text) including header and footer
     * 
     */
	public String getContents() {
		loadTranscript();
		StringBuffer buffer = new StringBuffer();
		buffer.append(m_headerText);
		buffer.append(m_contents);
		buffer.append(m_footerText);
		return buffer.toString();
	}
	
	
	/**
	 * Return the community ID associated with the transcript
	 * @return the community id
	 */
	public String getCommunityID() {
		if(m_communityID == null) {
			if(!m_parsedMetaInfo && m_existingFile) {
				parseMetaInfo();
			}
		}
		return m_communityID;
	}
	
	/** 
     * Get the contents this chat message without header/footer
     * @return the contents (typically HTML text) excluding header and footer
     * NOTE: This is typically for use for pasting into something which also contains 
     * the header and footer
     */
    public String getContentsWithoutHeaderAndFooter() {
		loadTranscript();
    	return m_contents != null ? (m_contents.length() > 0 ? m_contents.toString() : null) : null;
    }
	
    /** 
     * Get URL pointing to the contents of the message if can be so represented. 
     * @return the URL or <code>null</code>if contents cannot be represented by URL
     * @deprecated - Should not be used and eventually will be removed
     */
    public URL getContentsURL() {
    	URL result = null;
    	
    	if(m_file != null) {
    		try
    		{
    			result = m_file.toURL();
    		}
    		catch(Throwable t)
    		{
    			if(logger.isLoggable(Level.SEVERE)) {
    				logger.log(Level.SEVERE, 
    						    m_file.getAbsolutePath() +
    						   t);
    			}
    		}
    	}
    	
    	return result;
    }
	
    /** 
     * Get File containing the contents of the message. 
     * @return the File or <code>null</code>
     */
    public File getContentsFile() {
    	return m_file;
    }
    
  /*  *//**
     * Replace .\ references with a fully qualified and encoded URL
     * @return
     *//*
    private String unlocalizeURLs() {
		loadTranscript();
    	if ((m_file != null) && (m_contents != null)) {
    		HTMLUtils hu = new HTMLUtils();
    		hu.setAdapter(new HTMLUtilsAdapter() {
    			public String processParm(String strParm) {
    				if (strParm.startsWith("." + File.separator)) {  
    					strParm = FileUtils.getLocalUrl(m_file.getParent() + strParm.substring(1));
    				}
    				return strParm;
    			}
    		});
    		return hu.parseTagParm(m_contents.toString(), TAG_IMG, TAG_IMG_PARM);
    	} else {
    		if (m_contents != null) {
    			return m_contents.toString();
    		} else {
    			return null;
    		}
    	}
    }
    */
  
      /**
     * Set the footer to the given text
     * @param footer the footer text 
     */
    public void setFooter(String footer) {
		loadTranscript();
    	m_footerText = footer;
    }
    
    /**
     * Set the header to the given text
     * @param header the footer text 
     */
    public void setHeader(String header) {
		loadTranscript();
    	m_headerText = header;
    }
    
    /**
     * Set the community Id
     * @param communityID the community ID
     */
    protected void setCommunityID(String communityID) {
    	m_communityID = communityID;
    }
    
   
    
  
    
    /**
     * Parse the metadata in the current transcript and get various information,
     * including community ID if possible
     * 
     */
    public  ChatUserInfo parseMetaInfo() {
    	ChatUserInfo cui = new ChatUserInfo();
		loadTranscript();
    	if (!m_parsedMetaInfo && m_existingFile && (m_headerText != null)) {
    		m_parsedMetaInfo = true;
    		String host = null;
    		String providerId = null;
    		String username = null;
    		int headStartPos = m_headerText.indexOf("<head");  
    		if(headStartPos >= 0) {
    			int headEndPos = m_headerText.indexOf("</head>", headStartPos);  
    			if(headEndPos > headStartPos) {
    				String headElementText = 
    					m_headerText.substring(headStartPos, headEndPos + 7);
    				try
    				{
    					MetaInfoReader reader = new MetaInfoReader();
    					reader.parseMetaInfo(new InputSource(new StringReader(headElementText)));
    					host = reader.hostname;
    					providerId = reader.providerId;
    					username = reader.username;
    					//-------------------------------------------
    					// We may have gotten the fields like date 
    					// from another source (e.g filepath) already. 
    					// So we should only set the value from the 
    					// meta information only if we have a value
    					//-------------------------------------------
    					if(reader.initiator != null) {
    						m_initiator = reader.initiator;
    					}
    					if(reader.topic != null) {
    						m_chatTopic = reader.topic;
    					}
    					
    					if(reader.creationTime != null) {
    						m_date = reader.creationTime;
    					}
    					if(reader.lastActivityTime != null) {
    						m_lastActivity = reader.lastActivityTime;
    					}
    				}
    				catch(Throwable t)
    				{
    					host = null;
    					providerId = null;
    					username = null;
    				}
    				
    				
    				
    				cui.creationTime = m_date;
    				cui.initiator = m_initiator;
    				cui.lastActivityTime = m_lastActivity;
    				
    			}
    		}
    	}
    	
    	return cui;
    }
    
    /**
     * Parse the timestamps to get the latest activity time
     * 
     */
    private int parseChatCount() {
    	int count = 0;
		loadTranscript();
		if(!m_parsedTimestamps && (m_headerText != null)) {
    		m_parsedTimestamps = true;
    		int bodyStartPos = m_headerText.indexOf("<body");  
    		if(bodyStartPos >= 0) {
    			StringBuffer textBuffer = new StringBuffer();
    			textBuffer.append(m_headerText.substring(bodyStartPos));
    			textBuffer.append(m_contents);
    			int bodyEndPos = m_footerText.indexOf("</body>");  
    			if(bodyEndPos >= 0) {
    				textBuffer.append(m_footerText.substring(0, bodyEndPos + 7));
    				String bodyElementText = textBuffer.toString();
    				try
    				{
    					
    					
    				//	System.out.println(bodyElementText);
    					
//    					SAXParser sp = SAXParserFactory.newInstance().newSAXParser();
//    					ByteArrayInputStream is = new ByteArrayInputStream();
//    					sp.parse(is, new ChatCountParser(m_date));
//    					is.close();
    					ChatCountParser parser = new ChatCountParser(m_date);
    					count = parser.parse(new InputSource(new StringReader(bodyElementText)));
    				//	System.out.println("COUNT: "+ count);

    				}
    				catch(Throwable t)
    				{
    				}
    			}
    			textBuffer = null;
    		}
    	}
		
		return count;
    }
    
    /**
     * Parse the timestamps to get the latest activity time
     * 
     */
    private void parseTimestamps() {
		loadTranscript();
		
    	if(!m_parsedTimestamps && (m_headerText != null)) {
    		m_parsedTimestamps = true;
    		int bodyStartPos = m_headerText.indexOf("<body");  
    		if(bodyStartPos >= 0) {
    			StringBuffer textBuffer = new StringBuffer();
    			textBuffer.append(m_headerText.substring(bodyStartPos));
    			textBuffer.append(m_contents);
    			int bodyEndPos = m_footerText.indexOf("</body>");  
    			if(bodyEndPos >= 0) {
    				textBuffer.append(m_footerText.substring(0, bodyEndPos + 7));
    				String bodyElementText = textBuffer.toString();
    				System.out.println(bodyElementText);
    				try
    				{
    					ChatCountParser parser = new ChatCountParser(m_date);
    					parser.parse(new InputSource(new StringReader(bodyElementText)));
    					Date time = parser.getLastActivityTime();
    					
    					if(time != null) {  						
    						m_calendarLast.setTime(time);
    						m_calendarLast.set(m_calendarStart.get(Calendar.YEAR),
    								           m_calendarStart.get(Calendar.MONTH),
    								           m_calendarStart.get(Calendar.DAY_OF_MONTH));
    						//---------------------------------------
    						// Since we created the date part from
    						// the start time, if the last time is
    						// earlier, then assume we must have 
    						// rolled over the date, so increment 
    						// the date by 1.
    						//---------------------------------------
    						if(m_calendarLast.before(m_calendarStart)) {
    							m_calendarLast.add(Calendar.DATE, 1);
    						}
    						m_lastActivity = m_calendarLast.getTime();
    						
    					}

    				}
    				catch(Throwable t)
    				{
    				}
    			}
    			textBuffer = null;
    		}
    	}
    }
    
    
    
    
   
    
    /**
     * Update the last activity date
     */
    private void updateLastActivityDate() {
		loadTranscript();
    	m_lastActivity = new Date();
    	String strLastActivityMeta = createLastActivityMetaInfo();
    	
    	//
    	// Update our header with the new date
    	//
    	if (m_headerText != null) {
        	StringBuffer metaBuffer = new StringBuffer();
    		metaBuffer.append("<meta name=\"");  
    		metaBuffer.append(META_LAST_ACTIVITY_TIME);
    		int pos = m_headerText.indexOf(metaBuffer.toString());
    		if (pos != -1) {
    			int posend = m_headerText.indexOf('>', pos);
    			if (posend != -1) {
    				String strTemp;
    				strTemp = m_headerText.substring(0,	pos);
    				strTemp += strLastActivityMeta;
    				strTemp += m_headerText.substring(posend+1);
    				m_headerText = strTemp;
     			}
    		}
    	}
    	
    	//
    	// Update the meta array with the new date
    	//
    	if (m_metaInfo != null) {
    		int index = m_metaInfo.length - 1;
    		boolean found = false;
    		while (!found && (index >= 0)) {
    			if (m_metaInfo[index].indexOf(META_LAST_ACTIVITY_TIME) > 0) {
    				found = true;
    			} else {
    				--index;
    			}
    		}
    		if (index >= 0) {
    			m_metaInfo[index] = strLastActivityMeta;
    		}
    	}
    }
    
    /**
     * Create the metadata element for last activity.  We may do this multiple
     * times in multiple places, which is why we have a method for it
     * @return
     */
    private String createLastActivityMetaInfo() {
    	StringBuffer metaBuffer = new StringBuffer();
		metaBuffer.append("<meta name=\"");  
		metaBuffer.append(META_LAST_ACTIVITY_TIME);
		metaBuffer.append("\" content=\"");  
		metaBuffer.append(s_longDateFormat.format(m_lastActivity));
		metaBuffer.append("\"/>");  
		return metaBuffer.toString();
    }
    
    private Date convertTimestamp(Date startDate, String timestamp) {
    	Date result = null;
    	
    	//-----------------------------------------------------------
    	// First try 12 hour format and if that fails, try 24 hour
    	// format
    	//-----------------------------------------------------------
    	try
    	{
    		result = s_timeFormat12.parse(timestamp);
    		//	System.out.println("The result is1: "+result);
    		if(result != null) {  						
				m_calendarLast.setTime(result);
				m_calendarLast.set(m_calendarStart.get(Calendar.YEAR),
						           m_calendarStart.get(Calendar.MONTH),
						           m_calendarStart.get(Calendar.DAY_OF_MONTH));
				//---------------------------------------
				// Since we created the date part from
				// the start time, if the last time is
				// earlier, then assume we must have 
				// rolled over the date, so increment 
				// the date by 1.
				//---------------------------------------
				if(m_calendarLast.before(m_calendarStart)) {
					m_calendarLast.add(Calendar.DATE, 1);
				}
				m_lastActivity = m_calendarLast.getTime();
			//	System.out.println("m_lastActivity "+m_lastActivity);
			}
    		
    		
    	}
    	catch(ParseException pe) 
    	{
    		m_lastActivity = null;
    	}
    	if(result == null) {
    		try
    		{
    			result = s_timeFormat24.parse(timestamp);
    		//	System.out.println("The result is2: "+result);
        		if(result != null) {  						
    				m_calendarLast.setTime(result);
    				m_calendarLast.set(m_calendarStart.get(Calendar.YEAR),
    						           m_calendarStart.get(Calendar.MONTH),
    						           m_calendarStart.get(Calendar.DAY_OF_MONTH));
    				//---------------------------------------
    				// Since we created the date part from
    				// the start time, if the last time is
    				// earlier, then assume we must have 
    				// rolled over the date, so increment 
    				// the date by 1.
    				//---------------------------------------
    				if(m_calendarLast.before(m_calendarStart)) {
    					m_calendarLast.add(Calendar.DATE, 1);
    				}
    				m_lastActivity = m_calendarLast.getTime();
    			//	System.out.println("m_lastActivity "+m_lastActivity);
    			}
    			

    		}
    		catch(ParseException pe) 
    		{
    			m_lastActivity = null;
    		}
    	}
    	
    	return m_lastActivity;
    }
    
    /**
     * Determine whether the long time string has seconds (new format) or
     * not (old format)
     * @param longTimeString
     * @return
     */
    private static boolean timeHasSeconds(String longTimeString) {
    	boolean result = true;
    	int dashPos = longTimeString.indexOf('-');
		if(dashPos > 0) {
			int spacePos = longTimeString.indexOf(' ', dashPos);
			if(spacePos > 0) {
				if((spacePos - dashPos) == 5) {
					result = false;
				}
			}
		}
		
		return result;
    }

	/**
	 * @return the m_SavePersonProperties
	 */
	public boolean isSavePersonProperties() {
		return m_SavePersonProperties;
	}

	/**
	 * @param savePersonProperties the m_SavePersonProperties to set
	 */
	public void setSavePersonProperties(boolean savePersonProperties) {
		m_SavePersonProperties = savePersonProperties;
	}
	
  

	public boolean close() {
		// file logging does not use this, since it saves everytime an entry
		// is added to transcript.
		return false;
	}
	
	public class HTMLUtilsAdapter {

		public String processTag(String tag) {
			return tag;
		}
		
		public String processParm(String parm) {
			return parm;
		}
		
		public String processText(String text) {
			return text;
		}
	}

	public void append(String contents) {
		// TODO Auto-generated method stub
		
	}

	public boolean delete() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean save() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean saveToFile(File saveFile, int saveAsType) {
		// TODO Auto-generated method stub
		return false;
	}

	public void sendAsEmail(String to, String subject, String preTranscriptHTML) {
		// TODO Auto-generated method stub
		
	}
}


