package edu.columbia.logging.file;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Comparator;
import java.util.Date;
import java.util.Hashtable;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;

public class FilePersonHistory implements IChatPersonHistory{
	public static final String NWAY_FOLDER_NAME = "[multi-way]"; //$NON-NLS-1$
	public static final String ONE_ON_ONE_TRANSCRIPT_FOLDER_NAME = "yyyyMMdd"; //$NON-NLS-1$
	public static final String NWAY_TRANSCRIPT_FOLDER_NAME = "yyyyMMdd-HHmmss"; //$NON-NLS-1$
	private static Logger logger = Logger.getLogger(FilePersonHistory.class.getPackage().getName());
	
	private static Comparator fileComparator = new FileComparator();
	
	private static final class FileComparator implements Comparator {
		public int compare(Object arg0, Object arg1) {
			File file1 = (File) arg0;
			File file2 = (File) arg1;
			return file2.compareTo(file1);
		}
	}
	

	private String contactID;
	private String communityID;
	private boolean externalID = false;
	private String gwCommID = null;
	private String displayNameID = null;
//	private String serverID;
	private File personDir;
	private File[] dateDirs = null;
	private ArrayList transcripts = null;
	private boolean loadedDateDirs = false;
	
	public FilePersonHistory(String contactID, boolean externalID, String gwCommID, String communityID, String displayNameID, File personDir) {
		this.contactID = contactID;
		this.externalID = externalID;
		this.gwCommID = gwCommID;
		this.communityID = communityID;
		this.displayNameID = displayNameID;
		this.personDir = personDir;
	}
	
	/**
	 * Answer whether this is a valid FilePersonHistory directory
	 * @return <code>true</code> if found date directories, <code>false</code> otherwise
	 */
	public boolean isValid() {
		boolean bValid = false;
		if (personDir != null) {
			File propFile = new File(personDir.getAbsolutePath(), "ChatHistory.properties");
			if (propFile.exists() && propFile.canRead()) {
				bValid = true;
				
			} else {
				if (getDateDirs(personDir, true) != null) {
					bValid = true;
				}			
			}
		}
		return bValid;
	}

	/**
	 * Get the person's contact id
	 * @return String the contact id of the person
	 */
	public String getContactID() {
		return contactID;
	}
	
	/**
	 * Get the community id for these conversations
	 * @return String the community Id
	 */
	public String getCommunityID() {
		return communityID;
	}
	
	/**
	 * Return whether the contact ID is external
	 * @return true if external
	 */
	public boolean isExternalID() {
		return externalID;
	}
	
	public String getGWCommID() {
		return gwCommID;
	}
	
	/**
	 * @return the contact ID display name
	 */
	public String getDisplayName() {
		return displayNameID;
	}
	
	/**
	 * @return the person folder
	 */
	public File getFolder() {
		return personDir;
	}
	
	/**
	 * Sets the community id for these conversations
	 * @param communityID String
	 */
	protected void setCommunityID(String communityID) {
		this.communityID = communityID;
	}

	protected void setTranscripts(ArrayList list) {
		transcripts = list;
	}
	
	
	public IChatTranscript[] getAllTranscripts() {
		loadTranscripts();
		return (IChatTranscript[])transcripts.toArray(new IChatTranscript[0]);
	}
	
	/**
	 * Remove the given chat transcript from the history
	 * @return <code>true</code> if removed, <code>false</code> otherwise
	 */
	public boolean removeTranscript(IChatTranscript transcript) {
		boolean result = false;
				
		if(transcripts != null) {
			int found = -1;
			for(int index=0; (found < 0) && (index < transcripts.size()); ++index) {
				if(transcripts.get(index) == transcript) {
					found = index;
				}
			}
			if(found >= 0) {
				transcripts.remove(found);
				result = true;
			}
		}
		
		return result;
	}
	
	public static boolean isNWayFolder(File personDir) {
		if (personDir == null) return false;
		
		boolean bIsNWayFolder;
		if (personDir.getPath().indexOf(NWAY_FOLDER_NAME) > 0) {
			bIsNWayFolder = true;
		} else {
			bIsNWayFolder = false;
		}
		return bIsNWayFolder;
	}
	
    public static Date getTranscriptDate(File dateDir) {
    	Date result = null;
		if (dateDir.isDirectory() && dateDir.canRead()) {
			String dateName = null;
			int hour = 0;
			int minute = 0;
			int second = 0;
			if (isNWayFolder(dateDir)) {
				if (dateDir.getName().length() >= 17) {
					dateName = dateDir.getName().substring(0, 8);
					//
					// Format should be hhmmss (e.g. 10:05:15 is 100515)
					//
					try {
						hour = Integer.parseInt(dateDir.getName().substring(9, 11));
						minute = Integer.parseInt(dateDir.getName().substring(11, 13));
						second = Integer.parseInt(dateDir.getName().substring(13, 15));
						if ((hour < 0) || (hour > 23) || (minute < 0) || (minute > 59) || 
								(second < 0) || (second > 59)) {
							dateName = null;
						}
					} catch (Throwable t) {
						dateName = null;
					}
				}
			} else {
				if (dateDir.getName().length() == 8) {
					dateName = dateDir.getName();
				
				}
			}
			if (dateName != null) {
				//
				// Format should be yyyymmdd (e.g. March 13, 2006 is 20060313)
				//
				try {
					int year = Integer.parseInt(dateName.substring(0, 4));
					int month = Integer.parseInt(dateName.substring(4, 6));
					int day = Integer.parseInt(dateName.substring(6, 8));
					if (((year >= 1990) && (year <= 2030)) && 
						((month >= 1) && (month <= 12)) &&
						((day >= 1) && (day <= 31))) {
						Calendar cal = Calendar.getInstance();
						cal.set(year, month-1, day, hour, minute, second);
						result = cal.getTime();
					}
				} catch(Throwable t) {
				}
			}
		}
		
    	return result;
    }
    
	/**
	 * Locate the date directories under the given person directory
	 * @param personDir The person directory
	 * @param bCheckIfAnyValid Just check if there are any valid 
	 * @return File[] of date directories (sorted) or <code>null</code>if none
	 */
	public File[] getDateDirs(File personDir, boolean bCheckIfAnyValid) {
		if (personDir == null) return null;
		File[] result = null;
		File[] files = personDir.listFiles();
		if (files != null) {
			ArrayList validList = new ArrayList();
			for (int date = 0; date < files.length; date++) {
				File dateDir = files[date];
				if (getTranscriptDate(dateDir) != null) {
					validList.add(dateDir);
					//
					// Once we got a valid one we can stop checking
					//
					if (bCheckIfAnyValid) {
						break;
					}
				}
			}
			if (validList.size() > 0) {
				result = new File[validList.size()];
				validList.toArray(result);
				Arrays.sort(result, fileComparator);
			}
		}
		if (!bCheckIfAnyValid) {
			loadedDateDirs = true;
		}
		
		return result;
	}

	
	public void loadTranscripts() {
		SortedMap counttable = new TreeMap();
		if (transcripts == null) {
			transcripts = new ArrayList();
			if(!loadedDateDirs) {
				dateDirs = getDateDirs(personDir, false);
			}
			//-------------------------------------------------------
			// These should be directories for each date
			//-------------------------------------------------------
			if (dateDirs != null) {
				//boolean bIsNWayFolder = isNWayFolder(personDir);
				for (int date=0; date < dateDirs.length; date++) {
					File dateDir = dateDirs[date];
					String dateName = dateDir.getName();
					
					try 
					{
						Date tranDate = getTranscriptDate(dateDir);
						File[] fileTranscripts = dateDir.listFiles(new FileFilter() {
							public boolean accept(File pathname) {
								if (pathname.getName().endsWith(".html")) { 
									return true;
								}
								return false;
							};
						});
						for (int i=0; i<fileTranscripts.length; i++) {
							FileChatTranscript transcript = new FileChatTranscript(fileTranscripts[i], tranDate);
							//System.out.println(dateName);
//							transcript.setChatTopic(topic);
//							transcript.setChatPersonHistory(this); 
							
							int count = transcript.getPerDayChatCount(dateName,getContactID());
							ChatUserInfo cui = transcript.parseMetaInfo();
							//logger.info("Number of times i chatted with: "+getContactID() +"translated as "+tranDate + "is "+count);
							SortedMap date_entries = (SortedMap)ChatParserANDWriter.chat_data_by_date.get(dateName);
				            if (date_entries == null)
				            	ChatParserANDWriter.chat_data_by_date.put(dateName, date_entries=new TreeMap());
				            date_entries.put(getContactID()+"", new Integer(count));
				            
				            if(ChatParserANDWriter.totaldailychat.get(dateName) != null){
				            	int total_value = ((Integer)ChatParserANDWriter.totaldailychat.get(dateName)).intValue();
				            	total_value = total_value + count;
				            	ChatParserANDWriter.totaldailychat.put(dateName,new Integer(total_value));
				            }else{
				            	ChatParserANDWriter.totaldailychat.put(dateName,new Integer(count));
				            }
				            
				            
				            
				            if(ChatParserANDWriter.totaluserchat.get(getContactID()) != null){
				            	int total_value = ((Integer)ChatParserANDWriter.totaluserchat.get(getContactID())).intValue();
				            	total_value = total_value + count;
				            	ChatParserANDWriter.totaluserchat.put(getContactID(),new Integer(total_value));
				            	cui.total = total_value;
				            }else{
				            	ChatParserANDWriter.totaluserchat.put(getContactID(),new Integer(count));
				            	cui.total = count;
				            }		
							counttable.put(dateName, new Integer(count));
							ChatParserANDWriter.chat_data_by_user.put(getContactID(), counttable);
							ChatParserANDWriter.chatUserObject.put(getContactID(), cui);
//							ChatParserANDWriter.totalchatwithuser = ChatParserANDWriter.totalchatwithuser + count;
//							ChatParserANDWriter.totalchatnumber = ChatParserANDWriter.totalchatnumber + count;
							//transcripts.add(transcript);
						}
					}
					catch (Exception e) {
						if (logger.isLoggable(Level.FINE)) {
							logger.log(Level.FINE, "Error: "+ personDir.getAbsolutePath() + dateName + e);
						}
					}
				}
			}	
		}
		
	}
	
	
}